@@ -29,7 +29,6 @@ gboolean prefetch_enable = TRUE;
 gboolean prefetch_backpatch = TRUE;
 
 static prefetch_data_t *prefetch_data = NULL;
-static int              prefetch_shm_id = -1;
 
 static GHashTable *cant_prefetch = NULL;
 
@@ -265,7 +264,7 @@ static int prefetch_on_fork(void) {
 static void prefetch_hook_fork(void) {
 
   void *fork_addr =
-      GSIZE_TO_POINTER(gum_module_find_export_by_name(NULL, "fork"));
+      GSIZE_TO_POINTER(gum_module_find_global_export_by_name("fork"));
   intercept_hook(fork_addr, prefetch_on_fork, NULL);
 
 }
