@@ -8,13 +8,16 @@
 #include <list>
 #include <string>
 #include <fstream>
-#include <optional>
+
+#ifdef __has_include
+  #if __has_include(<optional>)
+    #include <optional>
+  #endif
+#endif
+
 #include <sys/time.h>
 
 #include "llvm/Config/llvm-config.h"
-#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR < 5
-typedef long double max_align_t;
-#endif
 
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/LegacyPassManager.h"
@@ -26,27 +29,16 @@ typedef long double max_align_t;
   #include "llvm/Transforms/IPO/PassManagerBuilder.h"
 #endif
 
-#if LLVM_VERSION_MAJOR > 3 || \
-    (LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR > 4)
-  #include "llvm/IR/DebugInfo.h"
-  #include "llvm/IR/CFG.h"
-#else
-  #include "llvm/DebugInfo.h"
-  #include "llvm/Support/CFG.h"
-#endif
+#include "llvm/IR/DebugInfo.h"
+#include "llvm/IR/CFG.h"
 
-#if LLVM_VERSION_MAJOR >= 11
-  #define MNAME M.getSourceFileName()
-  #define FMNAME F.getParent()->getSourceFileName()
-  #if LLVM_VERSION_MAJOR >= 16
+#define MNAME M.getSourceFileName()
+#define FMNAME F.getParent()->getSourceFileName()
+#if LLVM_VERSION_MAJOR >= 16
 // None becomes deprecated
 // the standard std::nullopt_t is recommended instead
 // from C++17 and onwards.
 constexpr std::nullopt_t None = std::nullopt;
-  #endif
-#else
-  #define MNAME std::string("")
-  #define FMNAME std::string("")
 #endif
 
 char *getBBName(const llvm::BasicBlock *BB);
