@@ -39,13 +39,9 @@
 #include "llvm/Config/llvm-config.h"
 #include "llvm/ADT/Statistic.h"
 #include "llvm/IR/IRBuilder.h"
-#if LLVM_VERSION_MAJOR >= 11                        /* use new pass manager */
-  #include "llvm/Passes/PassPlugin.h"
-  #include "llvm/Passes/PassBuilder.h"
-  #include "llvm/IR/PassManager.h"
-#else
-  #include "llvm/IR/LegacyPassManager.h"
-#endif
+#include "llvm/Passes/PassPlugin.h"
+#include "llvm/Passes/PassBuilder.h"
+#include "llvm/IR/PassManager.h"
 #include "llvm/IR/BasicBlock.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/DebugInfo.h"
@@ -72,7 +68,6 @@ using namespace llvm;
 
 namespace {
 
-#if LLVM_VERSION_MAJOR >= 11                        /* use new pass manager */
 class AFLdict2filePass : public PassInfoMixin<AFLdict2filePass> {
 
   std::ofstream of;
@@ -81,35 +76,16 @@ class AFLdict2filePass : public PassInfoMixin<AFLdict2filePass> {
  public:
   AFLdict2filePass() {
 
-#else
-
-class AFLdict2filePass : public ModulePass {
-
-  std::ofstream of;
-  void          dict2file(u8 *, u32);
-
- public:
-  static char ID;
-
-  AFLdict2filePass() : ModulePass(ID) {
-
-#endif
-
     if (getenv("AFL_DEBUG")) debug = 1;
 
   }
 
-#if LLVM_VERSION_MAJOR >= 11                        /* use new pass manager */
   PreservedAnalyses run(Module &M, ModuleAnalysisManager &MAM);
-#else
-  bool runOnModule(Module &M) override;
-#endif
 
 };
 
 }  // namespace
 
-#if LLVM_MAJOR >= 11
 extern "C" ::llvm::PassPluginLibraryInfo LLVM_ATTRIBUTE_WEAK
 llvmGetPassPluginInfo() {
 
@@ -117,24 +93,22 @@ llvmGetPassPluginInfo() {
           /* lambda to insert our pass into the pass pipeline. */
           [](PassBuilder &PB) {
 
-  #if LLVM_VERSION_MAJOR <= 13
-            using OptimizationLevel = typename PassBuilder::OptimizationLevel;
-  #endif
-            PB.registerOptimizerLastEPCallback(
-                [](ModulePassManager &MPM, OptimizationLevel OL) {
+            PB.registerOptimizerLastEPCallback([](ModulePassManager &MPM,
+                                                  OptimizationLevel  OL
+#if LLVM_VERSION_MAJOR >= 20
+                                                  ,
+                                                  ThinOrFullLTOPhase Phase
+#endif
+                                               ) {
 
-                  MPM.addPass(AFLdict2filePass());
+              MPM.addPass(AFLdict2filePass());
 
-                });
+            });
 
           }};
 
 }
 
-#else
-char AFLdict2filePass::ID = 0;
-#endif
-
 void AFLdict2filePass::dict2file(u8 *mem, u32 len) {
 
   u32  i, j, binary = 0;
@@ -174,14 +148,8 @@ void AFLdict2filePass::dict2file(u8 *mem, u32 len) {
 
 }
 
-#if LLVM_VERSION_MAJOR >= 11                        /* use new pass manager */
 PreservedAnalyses AFLdict2filePass::run(Module &M, ModuleAnalysisManager &MAM) {
 
-#else
-bool AFLdict2filePass::runOnModule(Module &M) {
-
-#endif
-
   DenseMap<Value *, std::string *> valueMap;
   char                            *ptr;
   int                              found = 0, handle_main = 1;
@@ -208,12 +176,8 @@ bool AFLdict2filePass::runOnModule(Module &M) {
 
   if (!ptr) {
 
-#if LLVM_VERSION_MAJOR >= 11                        /* use new pass manager */
     auto PA = PreservedAnalyses::all();
     return PA;
-#else
-    return true;
-#endif
 
   }
 
@@ -661,6 +625,13 @@ bool AFLdict2filePass::runOnModule(Module &M) {
             Value       *op2 = callInst->getArgOperand(2);
             ConstantInt *ilen = dyn_cast<ConstantInt>(op2);
 
+            if (!ilen) {
+
+              op2 = callInst->getArgOperand(1);
+              ilen = dyn_cast<ConstantInt>(op2);
+
+            }
+
             if (ilen) {
 
               uint64_t literalLength = optLen;
@@ -742,32 +713,7 @@ bool AFLdict2filePass::runOnModule(Module &M) {
 
   }
 
-#if LLVM_VERSION_MAJOR >= 11                        /* use new pass manager */
   auto PA = PreservedAnalyses::all();
   return PA;
-#else
-  return false;
-#endif
 
 }
-
-#if LLVM_VERSION_MAJOR < 11                         /* use old pass manager */
-static void registerAFLdict2filePass(const PassManagerBuilder &,
-                                     legacy::PassManagerBase &PM) {
-
-  PM.add(new AFLdict2filePass());
-
-}
-
-static RegisterPass<AFLdict2filePass> X("afl-dict2file",
-                                        "AFL++ dict2file instrumentation pass",
-                                        false, false);
-
-static RegisterStandardPasses RegisterAFLdict2filePass(
-    PassManagerBuilder::EP_OptimizerLast, registerAFLdict2filePass);
-
-static RegisterStandardPasses RegisterAFLdict2filePass0(
-    PassManagerBuilder::EP_EnabledOnOptLevel0, registerAFLdict2filePass);
-
-#endif
-
