@@ -30,30 +30,16 @@
 #include "llvm/Pass.h"
 #include "llvm/Support/raw_ostream.h"
 
-#if LLVM_MAJOR >= 11
-  #include "llvm/Passes/PassPlugin.h"
-  #include "llvm/Passes/PassBuilder.h"
-  #include "llvm/IR/PassManager.h"
-#else
-  #include "llvm/IR/LegacyPassManager.h"
-  #include "llvm/Transforms/IPO/PassManagerBuilder.h"
-#endif
+#include "llvm/Passes/PassPlugin.h"
+#include "llvm/Passes/PassBuilder.h"
+#include "llvm/IR/PassManager.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 #include "llvm/IR/Module.h"
-#if LLVM_VERSION_MAJOR >= 14                /* how about stable interfaces? */
-  #include "llvm/Passes/OptimizationLevel.h"
-#endif
+#include "llvm/Passes/OptimizationLevel.h"
 
 #include "llvm/IR/IRBuilder.h"
-#if LLVM_VERSION_MAJOR >= 4 || \
-    (LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR > 4)
-  #include "llvm/IR/Verifier.h"
-  #include "llvm/IR/DebugInfo.h"
-#else
-  #include "llvm/Analysis/Verifier.h"
-  #include "llvm/DebugInfo.h"
-  #define nullptr 0
-#endif
+#include "llvm/IR/Verifier.h"
+#include "llvm/IR/DebugInfo.h"
 
 using namespace llvm;
 #include "afl-llvm-common.h"
@@ -64,31 +50,17 @@ using namespace llvm;
 
 namespace {
 
-#if LLVM_MAJOR >= 11
 class SplitComparesTransform : public PassInfoMixin<SplitComparesTransform> {
 
  public:
   //  static char ID;
   SplitComparesTransform() : enableFPSplit(0) {
 
-#else
-class SplitComparesTransform : public ModulePass {
-
- public:
-  static char ID;
-  SplitComparesTransform() : ModulePass(ID), enableFPSplit(0) {
-
-#endif
-
     initInstrumentList();
 
   }
 
-#if LLVM_MAJOR >= 11
   PreservedAnalyses run(Module &M, ModuleAnalysisManager &MAM);
-#else
-  bool runOnModule(Module &M) override;
-#endif
 
  private:
   int enableFPSplit;
@@ -112,7 +84,7 @@ class SplitComparesTransform : public ModulePass {
   /// simplify a signed comparison (signed less or greater than)
   bool simplifySignedCompare(CmpInst *IcmpInst, Module &M,
                              CmpWorklist &worklist);
-  /// splits an icmp into nested icmps recursivly until target_bitwidth is
+  /// splits an icmp into nested icmps recursively until target_bitwidth is
   /// reached
   bool splitCompare(CmpInst *I, Module &M, CmpWorklist &worklist);
 
@@ -177,7 +149,6 @@ class SplitComparesTransform : public ModulePass {
 
 }  // namespace
 
-#if LLVM_MAJOR >= 11
 extern "C" ::llvm::PassPluginLibraryInfo LLVM_ATTRIBUTE_WEAK
 llvmGetPassPluginInfo() {
 
@@ -185,51 +156,26 @@ llvmGetPassPluginInfo() {
           /* lambda to insert our pass into the pass pipeline. */
           [](PassBuilder &PB) {
 
-  #if 1
-    #if LLVM_VERSION_MAJOR <= 13
-            using OptimizationLevel = typename PassBuilder::OptimizationLevel;
-    #endif
-    #if LLVM_VERSION_MAJOR >= 16
+#if LLVM_VERSION_MAJOR >= 16
             PB.registerOptimizerEarlyEPCallback(
-    #else
+#else
             PB.registerOptimizerLastEPCallback(
-    #endif
-                [](ModulePassManager &MPM, OptimizationLevel OL) {
+#endif
+                [](ModulePassManager &MPM, OptimizationLevel OL
+#if LLVM_VERSION_MAJOR >= 20
+                   ,
+                   ThinOrFullLTOPhase Phase
+#endif
+                ) {
 
                   MPM.addPass(SplitComparesTransform());
 
                 });
 
-  /* TODO LTO registration */
-  #else
-            using PipelineElement = typename PassBuilder::PipelineElement;
-            PB.registerPipelineParsingCallback([](StringRef          Name,
-                                                  ModulePassManager &MPM,
-                                                  ArrayRef<PipelineElement>) {
-
-              if (Name == "splitcompares") {
-
-                MPM.addPass(SplitComparesTransform());
-                return true;
-
-              } else {
-
-                return false;
-
-              }
-
-            });
-
-  #endif
-
           }};
 
 }
 
-#else
-char SplitComparesTransform::ID = 0;
-#endif
-
 /// This function splits FCMP instructions with xGE or xLE predicates into two
 /// FCMP instructions with predicate xGT or xLT and EQ
 bool SplitComparesTransform::simplifyFPCompares(Module &M) {
@@ -675,7 +621,7 @@ bool SplitComparesTransform::splitCompare(CmpInst *cmp_inst, Module &M,
 
   PHINode *PN = nullptr;
 
-  /* now we have to destinguish between == != and > < */
+  /* now we have to distinguish between == != and > < */
   switch (pred) {
 
     case CmpInst::ICMP_EQ:
@@ -763,7 +709,7 @@ bool SplitComparesTransform::splitCompare(CmpInst *cmp_inst, Module &M,
 
       } else {
 
-        // Never gonna appen
+        // Never gonna happen
         if (!be_quiet)
           fprintf(stderr,
                   "Error: split-compare: Equals or signed not removed: %d\n",
@@ -946,8 +892,6 @@ size_t SplitComparesTransform::splitFPCompares(Module &M) {
 
   LLVMContext &C = M.getContext();
 
-#if LLVM_VERSION_MAJOR >= 4 || \
-    (LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR > 7)
   const DataLayout &dl = M.getDataLayout();
 
   /* define unions with floating point and (sign, exponent, mantissa)  triples
@@ -962,8 +906,6 @@ size_t SplitComparesTransform::splitFPCompares(Module &M) {
 
   }
 
-#endif
-
   std::vector<CmpInst *> fcomps;
 
   /* get all EQ, NE, GT, and LT fcmps. if the other two
@@ -1431,14 +1373,14 @@ size_t SplitComparesTransform::splitFPCompares(Module &M) {
 
         case CmpInst::FCMP_UEQ:
         case CmpInst::FCMP_OEQ:
-          /* if the exponents are satifying the compare do a fraction cmp in
+          /* if the exponents are satisfying the compare do a fraction cmp in
            * middle_bb */
           BranchInst::Create(middle_bb, end_bb, icmp_exponent_result,
                              signequal2_bb);
           break;
         case CmpInst::FCMP_ONE:
         case CmpInst::FCMP_UNE:
-          /* if the exponents are satifying the compare do a fraction cmp in
+          /* if the exponents are satisfying the compare do a fraction cmp in
            * middle_bb */
           BranchInst::Create(end_bb, middle_bb, icmp_exponent_result,
                              signequal2_bb);
@@ -1705,15 +1647,9 @@ size_t SplitComparesTransform::splitFPCompares(Module &M) {
 
 }
 
-#if LLVM_MAJOR >= 11
 PreservedAnalyses SplitComparesTransform::run(Module                &M,
                                               ModuleAnalysisManager &MAM) {
 
-#else
-bool SplitComparesTransform::runOnModule(Module &M) {
-
-#endif
-
   if ((isatty(2) && getenv("AFL_QUIET") == NULL) ||
       getenv("AFL_DEBUG") != NULL) {
 
@@ -1818,12 +1754,8 @@ bool SplitComparesTransform::runOnModule(Module &M) {
   bool ret = count == 0 ? false : true;
 
   bool brokenDebug = false;
-  if (verifyModule(M, &errs()
-#if LLVM_VERSION_MAJOR >= 4 || \
-    (LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 9)
-                          ,
+  if (verifyModule(M, &errs(),
                    &brokenDebug  // 9th May 2016
-#endif
                    )) {
 
     reportError(
@@ -1848,7 +1780,6 @@ bool SplitComparesTransform::runOnModule(Module &M) {
 
   }
 
-#if LLVM_MAJOR >= 11
   /*  if (modified) {
 
       PA.abandon<XX_Manager>();
@@ -1859,36 +1790,5 @@ bool SplitComparesTransform::runOnModule(Module &M) {
     return PreservedAnalyses::all();
   else
     return PreservedAnalyses();
-#else
-  return ret;
-#endif
 
 }
-
-#if LLVM_MAJOR < 11                                 /* use old pass manager */
-
-static void registerSplitComparesPass(const PassManagerBuilder &,
-                                      legacy::PassManagerBase &PM) {
-
-  PM.add(new SplitComparesTransform());
-
-}
-
-static RegisterStandardPasses RegisterSplitComparesPass(
-    PassManagerBuilder::EP_OptimizerLast, registerSplitComparesPass);
-
-static RegisterStandardPasses RegisterSplitComparesTransPass0(
-    PassManagerBuilder::EP_EnabledOnOptLevel0, registerSplitComparesPass);
-
-  #if LLVM_VERSION_MAJOR >= 11
-static RegisterStandardPasses RegisterSplitComparesTransPassLTO(
-    PassManagerBuilder::EP_FullLinkTimeOptimizationLast,
-    registerSplitComparesPass);
-  #endif
-
-static RegisterPass<SplitComparesTransform> X("splitcompares",
-                                              "AFL++ split compares",
-                                              true /* Only looks at CFG */,
-                                              true /* Analysis Pass */);
-#endif
-
