@@ -63,6 +63,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/resource.h>
+#ifdef __linux__
+  #include <poll.h>
+  #include <unistd.h>
+#endif
 
 static afl_state_t *afl;
 
@@ -131,8 +135,9 @@ static const u8 count_class_binary[256] = {
 
 };
 
-static void kill_child() {
+static void kill_child(int signal) {
 
+  (void)signal;
   timed_out = 1;
   if (fsrv->child_pid > 0) {
 
@@ -158,10 +163,11 @@ void show_stats(afl_state_t *afl) {
 
 }
 
-void update_bitmap_score(afl_state_t *afl, struct queue_entry *q) {
+void update_bitmap_score(afl_state_t *afl, struct queue_entry *q, bool x) {
 
   (void)afl;
   (void)q;
+  (void)x;
 
 }
 
@@ -535,10 +541,74 @@ static u32 read_file(u8 *in_file) {
 }
 
 #ifdef __linux__
-/* Execute the target application with an empty input (in Nyx mode). */
+  #define NYX_STDIN_TMP_BUFFER_SIZE 256
+
+size_t read_stdin_input(u8 **out) {
+
+  size_t len = 0;
+  u8    *input = NULL;
+
+  if (!isatty(STDIN_FILENO)) {   /* only process input when passed via pipe */
+
+    struct pollfd pfd = {.fd = STDIN_FILENO, .events = POLLIN};
+    int           ready = poll(&pfd, 1, 0);
+
+    if (ready == -1) {
+
+      FATAL("%s: poll failed\n", __func__);
+
+    } else if (pfd.revents & POLLIN) {  // check if there is any data on stdin
+
+      u8     tmp[NYX_STDIN_TMP_BUFFER_SIZE];
+      size_t capacity = NYX_STDIN_TMP_BUFFER_SIZE;
+      size_t bytes_read;
+      input = malloc(capacity);
+
+      /* copy input from stdin into a buffer to pass it to Nyx */
+      while ((bytes_read = read(STDIN_FILENO, tmp, NYX_STDIN_TMP_BUFFER_SIZE)) >
+             0) {
+
+        /* resize buffer if needed */
+        if (len + bytes_read > capacity) {
+
+          while (len + bytes_read > capacity)
+            capacity *= 2;
+
+          u8 *new_input = realloc(input, capacity);
+          if (!new_input) {
+
+            free(input);
+            FATAL("%s: realloc failed\n", __func__);
+
+          }
+
+          input = new_input;
+
+        }
+
+        memcpy(input + len, tmp, bytes_read);
+        len += bytes_read;
+
+      }
+
+      *out = input;
+
+    }
+
+  }
+
+  return len;
+
+}
+
+/* Execute the target application in Nyx mode with piped input from stdin
+ * (passed as a buffer). */
 static void showmap_run_target_nyx_mode(afl_forkserver_t *fsrv) {
 
-  afl_fsrv_write_to_testcase(fsrv, NULL, 0);
+  u8    *input = NULL;
+  size_t len = read_stdin_input(&input);
+
+  afl_fsrv_write_to_testcase(fsrv, input, len);
 
   if (afl_fsrv_run_target(fsrv, fsrv->exec_tmout, &stop_soon) ==
       FSRV_RUN_ERROR) {
@@ -547,6 +617,8 @@ static void showmap_run_target_nyx_mode(afl_forkserver_t *fsrv) {
 
   }
 
+  if (input != NULL) { free(input); }
+
 }
 
 #endif
@@ -712,55 +784,8 @@ static void handle_stop_sig(int sig) {
 
 static void set_up_environment(afl_forkserver_t *fsrv, char **argv) {
 
-  char *afl_preload;
-  char *frida_afl_preload = NULL;
-
   set_sanitizer_defaults();
-
-  if (get_afl_env("AFL_PRELOAD")) {
-
-    if (fsrv->qemu_mode) {
-
-      /* afl-qemu-trace takes care of converting AFL_PRELOAD. */
-
-    } else if (fsrv->frida_mode) {
-
-      afl_preload = getenv("AFL_PRELOAD");
-      u8 *frida_binary = find_afl_binary(argv[0], "afl-frida-trace.so");
-      if (afl_preload) {
-
-        frida_afl_preload = alloc_printf("%s:%s", afl_preload, frida_binary);
-
-      } else {
-
-        frida_afl_preload = alloc_printf("%s", frida_binary);
-
-      }
-
-      ck_free(frida_binary);
-
-      setenv("LD_PRELOAD", frida_afl_preload, 1);
-      setenv("DYLD_INSERT_LIBRARIES", frida_afl_preload, 1);
-
-    } else {
-
-      /* CoreSight mode uses the default behavior. */
-
-      setenv("LD_PRELOAD", getenv("AFL_PRELOAD"), 1);
-      setenv("DYLD_INSERT_LIBRARIES", getenv("AFL_PRELOAD"), 1);
-
-    }
-
-  } else if (fsrv->frida_mode) {
-
-    u8 *frida_binary = find_afl_binary(argv[0], "afl-frida-trace.so");
-    setenv("LD_PRELOAD", frida_binary, 1);
-    setenv("DYLD_INSERT_LIBRARIES", frida_binary, 1);
-    ck_free(frida_binary);
-
-  }
-
-  if (frida_afl_preload) { ck_free(frida_afl_preload); }
+  afl_fsrv_setup_preload(fsrv, argv[0]);
 
 }
 
@@ -1253,8 +1278,6 @@ int main(int argc, char **argv_orig, char **envp) {
 
         break;
 
-      /* FIXME: We want to use -P for consistency, but it is already unsed for
-       * undocumenetd feature "Another afl-cmin specific feature." */
       case 'A':                                           /* CoreSight mode */
 
 #if !defined(__aarch64__) || !defined(__linux__)
@@ -1301,7 +1324,7 @@ int main(int argc, char **argv_orig, char **envp) {
         break;
 #else
       case 'X':
-        FATAL("Nyx mode is only availabe on linux...");
+        FATAL("Nyx mode is only available on linux...");
         break;
 #endif
 
@@ -1395,7 +1418,7 @@ int main(int argc, char **argv_orig, char **envp) {
   fsrv->target_path = find_binary(argv[optind]);
 #endif
 
-  fsrv->trace_bits = afl_shm_init(&shm, map_size, 0);
+  fsrv->trace_bits = afl_shm_init(&shm, map_size, 0, DEFAULT_PERMISSION, -1);
 
   if (!quiet_mode) {
 
@@ -1534,9 +1557,16 @@ int main(int argc, char **argv_orig, char **envp) {
   shm_fuzz->cmplog_mode = 0;
   atexit(at_exit_handler);
 
-  u8 *map = afl_shm_init(shm_fuzz, MAX_FILE + sizeof(u32), 1);
+  size_t shm_fuzz_map_size = SHM_FUZZ_MAP_SIZE_DEFAULT;
+  u8    *map = afl_shm_init(shm_fuzz, SHM_FUZZ_MAP_SIZE_DEFAULT, 1,
+                            DEFAULT_PERMISSION, -1);
   shm_fuzz->shmemfuzz_mode = true;
   if (!map) { FATAL("BUG: Zero return from afl_shm_init."); }
+
+  u8 *shm_fuzz_map_size_str = alloc_printf("%zu", shm_fuzz_map_size);
+  setenv(SHM_FUZZ_MAP_SIZE_ENV_VAR, shm_fuzz_map_size_str, 1);
+  ck_free(shm_fuzz_map_size_str);
+
 #ifdef USEMMAP
   setenv(SHM_FUZZ_ENV_VAR, shm_fuzz->g_shm_file_path, 1);
 #else
@@ -1583,15 +1613,26 @@ int main(int argc, char **argv_orig, char **envp) {
 
       // only reinitialize when it makes sense
       if (map_size < new_map_size ||
-          (new_map_size > map_size && new_map_size - map_size > MAP_SIZE)) {
+          (new_map_size < map_size && map_size - new_map_size > MAP_SIZE)) {
 
         if (!be_quiet)
           ACTF("Acquired new map size for target: %u bytes\n", new_map_size);
 
-        afl_shm_deinit(&shm);
-        afl_fsrv_kill(fsrv);
-        fsrv->map_size = new_map_size;
-        fsrv->trace_bits = afl_shm_init(&shm, new_map_size, 0);
+#ifdef __linux__
+        /* no need to terminate the nyx runner */
+        if (!fsrv->nyx_mode) {
+
+#endif
+          afl_shm_deinit(&shm);
+          afl_fsrv_kill(fsrv);
+          fsrv->map_size = new_map_size;
+          fsrv->trace_bits =
+              afl_shm_init(&shm, new_map_size, 0, DEFAULT_PERMISSION, -1);
+#ifdef __linux__
+
+        }
+
+#endif
 
       }
 
@@ -1678,7 +1719,7 @@ int main(int argc, char **argv_orig, char **envp) {
     } else {
 
       if ((coverage_map = (u8 *)malloc(map_size + 64)) == NULL)
-        FATAL("coult not grab memory");
+        FATAL("could not grab memory");
       edges_only = false;
 
     }
