@@ -65,7 +65,7 @@ if [ ! -f "../afl-showmap" ]; then
 
 fi
 
-PYTHONBIN=`command -v python3 || command -v python || command -v python2 || echo python3`
+PYTHONBIN=`command -v python3 || command -v python || echo python3`
 MAKECMD=make
 TARCMD=tar
 
@@ -116,7 +116,7 @@ for i in $PYTHONBIN automake autoconf git $MAKECMD $TARCMD; do
 done
 
 # some python version should be available now
-PYTHONS="`command -v python3` `command -v python` `command -v python2`"
+PYTHONS="`command -v python3` `command -v python`"
 PIP_FOUND=0
 for PYTHON in $PYTHONS ; do
 
@@ -199,26 +199,26 @@ echo "[+] Build process successful!"
 
 echo "[*] Installing Unicorn python bindings..."
 XOPT=
-$PYTHONBIN -m pip install --help 2>/dev/null | grep -q break-system-packages && XOPT=--break-system-packages
+#$PYTHONBIN -m pip install --help 2>/dev/null | grep -q break-system-packages && XOPT=--break-system-packages
 cd unicorn/bindings/python || exit 1
 if [ -z "$VIRTUAL_ENV" ]; then
   echo "[*] Info: Installing python unicornafl using --user"
-  THREADS=$CORES $PYTHONBIN -m pip install --user $XOPT --force .|| exit 1
+  THREADS=$CORES $PYTHONBIN setup.py install --user $XOPT --force || exit 1
 else
   echo "[*] Info: Installing python unicornafl to virtualenv: $VIRTUAL_ENV"
-  THREADS=$CORES $PYTHONBIN -m pip install --force .|| exit 1
+  THREADS=$CORES $PYTHONBIN setup.py install --force || exit 1
 fi
 cd ../../../
 echo "[*] Installing Unicornafl python bindings..."
 cd bindings/python || exit 1
 if [ -z "$VIRTUAL_ENV" ]; then
   echo "[*] Info: Installing python unicornafl using --user"
-  THREADS=$CORES $PYTHONBIN -m pip install --user $XOPT --force .|| exit 1
+  THREADS=$CORES $PYTHONBIN setup.py install --user $XOPT --force || exit 1
 else
   echo "[*] Info: Installing python unicornafl to virtualenv: $VIRTUAL_ENV"
-  THREADS=$CORES $PYTHONBIN -m pip install --force .|| exit 1
+  THREADS=$CORES $PYTHONBIN setup.py install --force || exit 1
 fi
-echo '[*] If needed, you can (re)install the bindings in `./unicornafl/bindings/python` using `pip install --force .`'
+echo '[*] If needed, you can (re)install the bindings in `./unicornafl/bindings/python` using `python setup.py install --force .`'
 
 cd ../../ || exit 1
 
